///////////////////////////////////////////////////////////////////////////
//
//   Copyright 2010 Alberto González Palomo
//   Author: Alberto González Palomo - http://sentido-labs.com/
//
//   This file is part of Tutorial Toolkit, abbreviated as TTK.
//
//   TTK is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 3 of the License, or
//   (at your option) any later version.
//
//   TTK is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with TTK; if not, see <http://www.gnu.org/licenses/>.
//
/////////////////////////////////////////////////////////////////////////////

package com.sentidolabs.ttk.example;

import javax.swing.*;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import com.sentidolabs.ttk.Stencil;
import com.sentidolabs.ttk.CallOut;
import com.sentidolabs.ttk.PaperNoteCallOut;
import com.sentidolabs.ttk.ThoughtBubbleCallOut;
import com.sentidolabs.ttk.CircleHighlight;

public class Complete implements ActionListener
{
    private JFrame window;
    private Stencil stencil;
    private JButton buttonStart, buttonEast, button1, button2, button3, button4, button5, button6, buttonInternal;
    private JScrollPane listScrollPane;
    private JDesktopPane desktop;
    private int state;
    
    public Complete()
    {
        JFrame.setDefaultLookAndFeelDecorated(false);
        window = new JFrame("Tutorial Toolkit Example");
        window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        window.setSize(800,600);
        buildExampleGUI(window);
        
        stencil = null;
        state = 0;
        
        window.setVisible(true);
    }
    
    private void next()
    {
        CallOut callOut;
        JButton button;
        switch (state) {
        case 0:
            if (null == stencil) stencil = new Stencil(window);
            stencil.addHoleFor(buttonEast);
            stencil.addHoleFor(button1);
            stencil.addHoleFor(button5);
            stencil.addHoleFor(buttonInternal);
            stencil.addHoleFor(desktop);
            stencil.addCoverFor(buttonInternal);
            
            callOut = new PaperNoteCallOut();
            callOut.setContent("<h1>Take note</h1><p>A note's content can be<br/>set from an HTML document<br/>or fragment (as this one<br/>including the image below),<br/>or as a Java component<br/>you provide.<center><img src='http://sentido-labs.com/en/sentido-labs.png'/></center>If you assign targets to it,<br/>there will be an arrow<br/>to each of them.</p>");
            button = new JButton("Next");
            button.addActionListener(this);
            callOut.addTarget(buttonEast);
            callOut.addTarget(button1);
            callOut.addTarget(button5);
            callOut.addTarget(buttonInternal);
            callOut.add(button);
            stencil.add(callOut);
            button.requestFocusInWindow();
            state = 1;
            break;
        case 1:
            callOut = new PaperNoteCallOut();
            callOut.setContent("<h1>More about notes</h1><p>The targets do not<br/>need to be the same<br/>as the holes,<br/>because we might want<br/>to talk about the list<br/>without letting the user<br/>interact with it for now.</p>");
            button = new JButton("I see");
            button.addActionListener(this);
            callOut.addTarget(listScrollPane);
            callOut.addHighlightTarget(listScrollPane);
            callOut.add(button);
            stencil.removeAll();
            stencil.add(callOut);
            button.requestFocusInWindow();
            state = 2;
            break;
        case 2:
            callOut = new ThoughtBubbleCallOut();
            callOut.setContent("<h1>Bubble thoughts</h1><table border='0'><tr><td><img src='http://www.lkl.ac.uk/cms/images/stories/Project_pics/migen-pond-cloud.gif' width='150' height='100'/></td><td>Thought bubble call-outs link<br/>to their targets through<br/>a chain of bubbles.</td></tr></table><b>How can I add a circle highlight<br/>to the button '6'?</b>");
            button = new JButton("Tell me");
            button.addActionListener(this);
            callOut.add(button);
            callOut.addTarget(buttonEast);
            callOut.addTarget(button1);
            callOut.addTarget(button5);
            stencil.removeAll();
            stencil.add(callOut);
            button.requestFocusInWindow();
            state = 3;
            break;
        case 3:
            callOut = new ThoughtBubbleCallOut();
            callOut.setContent("<h1>Bubble thoughts</h1><p>Use the function <tt>addHighlightTarget()</tt><br/>like in the previous highlight example,<br/>but then set the highlight style<br/>for this call-out to be<br/>a circle with<br/><tt>setHighlight(new CircleHighlight())</tt>.</p>");
            button = new JButton("Alright, I got it");
            button.addActionListener(this);
            callOut.add(button);
            callOut.addTarget(buttonEast);
            callOut.addTarget(button1);
            callOut.addTarget(button5);
            callOut.addHighlightTarget(button6);
            callOut.setHighlight(new CircleHighlight());
            stencil.removeAll();
            stencil.add(callOut);
            button.requestFocusInWindow();
            state = 4;
            break;
        case 4:
            stencil.detach();
            stencil = null;
            state = 0;
            break;
        default:
            System.err.println("Unexpected state " + state);
        }
    }
    
    public void actionPerformed(ActionEvent e)
    {
        String command = e.getActionCommand();
        if ("quit".equals(command)) {
            System.exit(0);
        }
        else {
            next();
        }
    }
    
    private void buildExampleGUI(JFrame frame)
    {
        JList list = new JList();
        listScrollPane = new JScrollPane(list);
        String[] listData = {
            "City of London",
            "City of Westminster",
            "Kensington and Chelsea",
            "Hammersmith and Fulham",
            "Wandsworth",
            "Lambeth",
            "Southwark",
            "Tower Hamlets",
            "Hackney",
            "Islington",
            "Camden",
            "Brent",
            "Ealing",
            "Hounslow",
            "Richmond upon Thames",
            "Kingston upon Thames",
            "Merton",
            "Sutton",
            "Croydon",
            "Bromley",
            "Lewisham",
            "Greenwich",
            "Bexley",
            "Havering",
            "Barking and Dagenham",
            "Redbridge",
            "Newham",
            "Waltham Forest",
            "Haringey",
            "Enfield",
            "Barnet",
            "Harrow",
            "Hillingdon"
        };
        list.setListData(listData);
        list.setVisibleRowCount(10);
        list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        JPanel controls = new JPanel();
        controls.add(button1 = new JButton("1"));
        controls.add(button2 = new JButton("2"));
        controls.add(button3 = new JButton("3"));
        controls.add(listScrollPane);
        controls.add(button4 = new JButton("4"));
        controls.add(button5 = new JButton("5"));
        controls.add(button6 = new JButton("6"));
        frame.getContentPane().add(controls, BorderLayout.SOUTH);
        frame.getContentPane().add(new JButton("Top"), BorderLayout.NORTH);
        frame.getContentPane().add(buttonEast = new JButton("East"), BorderLayout.EAST);
        frame.getContentPane().add(new JButton("West"), BorderLayout.WEST);
        desktop = new JDesktopPane();
        JInternalFrame desktopWindow = new JInternalFrame("Internal", true);
        desktopWindow.add(buttonStart = new JButton("Start tutorial"), BorderLayout.CENTER);
        desktopWindow.setBounds(0, 0, 200, 300);
        desktopWindow.setVisible(true);
        JInternalFrame secondDesktopWindow = new JInternalFrame("Second internal", true);
        secondDesktopWindow.add(new JLabel("Internal window"), BorderLayout.CENTER);
        secondDesktopWindow.setBounds(0, 0, 150, 200);
        secondDesktopWindow.setVisible(true);
        JPanel internalButtons = new JPanel();
        desktopWindow.add(internalButtons, BorderLayout.SOUTH);
        internalButtons.setLayout(new BoxLayout(internalButtons, BoxLayout.X_AXIS));
        internalButtons.add(buttonInternal = new JButton("One button"));
        internalButtons.add(new JButton("Another button"));
        internalButtons.add(new JButton("Yet another button"));
        desktop.add(desktopWindow);
        desktop.add(secondDesktopWindow);
        frame.getContentPane().add(desktop, BorderLayout.CENTER);
        buttonStart.addActionListener(this);
        
        JMenuBar menuBar = new JMenuBar();
        JMenu menu = new JMenu("Start");
        menuBar.add(menu);
        JMenuItem item;
        menu.add(item = new JMenuItem("Tutorial"));
        item.addActionListener(this);
        menu.addSeparator();
        menu.add(item = new JMenuItem("Quit"));
        item.addActionListener(this);
        item.setActionCommand("quit");
        frame.setJMenuBar(menuBar);
    }
    
    public static void main(String[] args)
    {
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
                public void run() 
                {
                    Complete example = new Complete();
                }
            });
    }
}
